﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.PluginInfrastructure;

namespace HeuristicLab.Persistence {
  [NonDiscoverableType]
  internal sealed class Index<T> where T : class {
    private readonly Dictionary<T, uint> indexes;
    private readonly Dictionary<uint, T> values;
    private uint nextIndex;

    public Index() {
      this.indexes = new Dictionary<T, uint>();
      this.values = new Dictionary<uint, T>();
      nextIndex = 1;
    }

    public Index(IEnumerable<T> values) : this() {
      foreach (var value in values) {
        this.indexes.Add(value, nextIndex);
        this.values.Add(nextIndex, value);
        nextIndex++;
      }
    }

    public uint GetIndex(T value) {
      uint index = 0;
      if (value != null && !indexes.TryGetValue(value, out index)) {
        index = nextIndex;
        nextIndex++;
        indexes.Add(value, index);
        values.Add(index, value);
      }
      return index;
    }

    public T GetValue(uint index) {
      return index == 0 ? null : values[index];
    }

    public IEnumerable<T> GetValues() {
      return values.Values;
    }
  }
}
